<?php
// $Id: flexicharge.admin.inc,v 1.1.2.11 2007/08/13 19:23:34 brmassa Exp $

/**
 * List all charges and new charge options
 */
function _flexicharge_charges_overview() {

  // First we display a list of currently configured charges.
//   drupal_set_title(t('Flexicharge Overview'));
  $operators = _flexicharge_operators('list');
  $charge_list = array();

  if ($charges = _flexicharge_load_charges()) {
    foreach ($charges AS $charge) {
      $row['name'] = l($charge->display, 'admin/ecsettings/flexicharge/edit/'. $charge->chid);
      $row['type'] = $charge->provider;
      $row['weight'] = $charge->position;
      $row['delete'] = l('delete', 'admin/ecsettings/flexicharge/delete/'. $charge->chid);
      array_push($charge_list, $row);
    }
    $output = theme('fieldset',
                  array(
                    '#title' => 'Currently configured charges',
                    '#children' => theme('table', array('Name', 'Type', 'Weight', 'Operations'), $charge_list),
                    '#collapsible' => true,
                    '#collapsed' => false)
                  );
  }
  else {
    $output = '<p>'. t('There is no charges yet.') .'</p>';
  }

  // Second we add a fieldset containing links to create new charges.
  $charge_methods = _flexicharge_charge_methods();
  foreach ($charge_methods as $provider => $methods) {
    foreach (element_children($methods) as $method) {
      $charge_menu[] = l($methods[$method]['#title'], 'admin/ecsettings/flexicharge/add', null , "provider=$provider&method=$method");
    }
  }
  $output .= theme('fieldset',
                array(
                  '#title' => 'Add a new charge',
                  '#children' => theme('item_list', $charge_menu)));
 return $output;
}

/**
 * Creating/editing charge form
 */
function flexicharge_admin_form() {
  $charge_methods = _flexicharge_charge_methods();

  switch (arg(3)) {
    case 'add':
      drupal_set_title(t('New Flexicharge'));
      $op = 'add';
      $chid = 0;
      $provider = $_GET['provider'];
      $method = $_GET['method'];
      break;
    case 'edit':
      $op = 'edit';
      $chid = arg(4);
      $charge = _flexicharge_load_charges($chid);
      $provider = $charge->provider; // convenience
      $method = $charge->method; // convenience
      drupal_set_title(t('Edit') .' '. check_plain($charge->display));
      break;
  }

  $form = array();
  $form['module'] = array(
    '#type' => 'fieldset',
    '#title' => 'Flexicharge defaults',
    '#description' => 'Charge description goes here',
    '#collapsible' => true,
    '#collapsed' => false,
    '#tree' => true
  );
  $form['module']['chid'] = array(
    '#type' => 'value',
    '#value' => $chid
  );
  $form['module']['provider'] = array(
    '#type' => 'value',
    '#value' => $provider
  );
  $form['module']['method'] = array(
    '#type' => 'value',
    '#value' => $method
  );
  $form['module']['display'] = array(
    '#type' => 'textfield',
    '#required' => true,
    '#title' => t('Display label'),
    '#description' => t('This is the description for the review screen and invoice.'),
    '#default_value' => ($chid) ? $charge->display : $charge_methods[$provider][$method]['#title'],
    '#size' => 20,
  );
  $operators[0] = '--';
  $operators = _flexicharge_operators('list');

  $form['module']['elements'] = array();

  $form['module']['operator'] = array(
    '#type' => 'select',
    '#title' => t('Operator'),
    '#default_value' => ($chid) ? $charge->operator : FLEXICHARGE_ADD,
    '#options' => $operators,
    );
  $form['module']['rate'] = array(
    '#type' => 'textfield',
    '#title' => t('Charge rate'),
    '#default_value' => ($chid) ? $charge->rate : 0,
    '#description' => t('Use positive or negative values. You can specify percent like 10 or -20, use the operator to specify percentage.'),
    '#size' => 8,
  );
  $form['module']['position'] = array(
    '#type' => 'textfield',
    '#required' => true,
    '#title' => t('Calculation order'),
    '#description' => t('Position of this charge in the flexicharge calculations.'),
    '#default_value' => ($chid) ? $charge->position : 1,
    '#size' => 5,
  );
  $form['module']['subtotal_before'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display a subtotal before?'),
    '#default_value' => ($chid) ? $charge->subtotal_before : 0,
  );
  $form['module']['subtotal_after'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display a subtotal after?'),
    '#default_value' => ($chid) ? $charge->subtotal_after : 0,
  );
  $form['module']['already_added'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display this amount as \'included\', but don\'t add it to the subtotal.'),
    '#default_value' => ($chid) ? $charge->already_added : 0,
  );
  $form['module']['hide_if_zero'] = array(
    '#type' => 'checkbox',
    '#title' => t('Suppress this row in the cart if the total charge = 0.'),
    '#default_value' => ($chid) ? $charge->hide_if_zero : 0,
  );
  $ptypes = product_get_ptypes('names');
  $form['module']['ptypes'] = array(
    '#type' => 'select',
    '#title' => t('Product types'),
    '#options' => $ptypes,
    '#multiple' => true,
    '#default_value' => empty($charge->ptypes) ? 0 : $charge->ptypes,
    '#description' => 'Please choose the products that this charge applies to. No selection indicates all products. Charge will apply only once, if any product in the cart matches this list.',
  );
  $all_roles = user_roles();
  $form['module']['roles'] = array(
    '#type' => 'select',
    '#title' => t('Roles'),
    '#options' => $all_roles,
    '#multiple' => true,
    '#default_value' => (($charge->roles) ? $charge->roles : 0),
    '#description' => 'Please select roles that this charge applies to. No selection indicates all roles. Charge will apply only once, if user is in any selected role.',
  );

  // Get the form elements required by the charge provider
  $f = $provider .'_flexicharge_form_elements';
  if (function_exists($f)) {
    //$form passed by array
    $f($form, $op, $method);
    $form['elements']['#tree'] = true;
  }

  // If there are custom form elements - apply them from the database values
  if (arg(3) == 'edit' && is_array($charge->settings)) {
    foreach ($charge->settings as $e => $s) {
      foreach ($form['elements'] AS $f => $fieldset) {
        if (isset($form['elements'][$f][$e])) {
          $form['elements'][$f][$e]['#default_value'] = $s;
        }
      }
    }
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save charge'),
  );

  return $form;
}

/**
 * Implementation of hook_submit().
 *
 * Creating/editing charge form
 */
function flexicharge_admin_form_submit($form_id, $form) {
  // Get the form elements required by the charge provider
  $f = $provider .'_flexicharge_form_validate';
  if (function_exists($f)) {
    $form = $f($form_id, $form);
  }

  $settings = array();
  if (isset($form['elements'])) {
    foreach ($form['elements'] AS $fieldset) {
      $settings = array_merge($fieldset, $settings);
    }
    $settings = serialize($settings);
  }
  else {
    $settings = '';
  }

  $ptypes = serialize($form['module']['ptypes']);
  $roles = serialize($form['module']['roles']);

  if (!empty($form['module']['chid'])) {
    db_query('UPDATE {ec_flexicharge} SET display = "%s", operator = %d, rate = %f,
      position = %d, subtotal_before = %d, subtotal_after = %d, already_added = %d,
      hide_if_zero = %d, ptypes = "%s", roles = "%s", settings = "%s" WHERE chid = %d',
      $form['module']['display'], $form['module']['operator'],
      $form['module']['rate'], $form['module']['position'],
      $form['module']['subtotal_before'], $form['module']['subtotal_after'],
      $form['module']['already_added'], $form['module']['hide_if_zero'],
      $ptypes, $roles, $settings, $form['module']['chid']);
  }
  else {
    db_query('INSERT INTO {ec_flexicharge} ( provider, method, display, operator,
      rate, position, subtotal_before, subtotal_after, already_added, hide_if_zero,
      ptypes, roles, settings) VALUES ("%s", "%s", "%s", %d, %f, %d, %d, %d, %d,
      %d, "%s", "%s", "%s")', $form['module']['provider'], $form['module']['method'],
      $form['module']['display'], $form['module']['operator'],
      $form['module']['rate'], $form['module']['position'],
      $form['module']['subtotal_before'], $form['module']['subtotal_after'],
      $form['module']['already_added'], $form['module']['hide_if_zero'],
      $ptypes, $roles, $settings);
  }

  drupal_set_message(t('The charge has been saved.'));
  drupal_goto('admin/ecsettings/flexicharge');
}

/**
 * Delete a charge
 */
function _flexicharge_charge_delete() {
  db_query('DELETE FROM {ec_flexicharge} WHERE chid = %d', arg(4));
  drupal_goto('admin/ecsettings/flexicharge');
}

