<?php
/**
 * $Id: invoice.admin.inc,v 1.1.2.7 2007/07/26 07:00:51 brmassa Exp $
 * @file invoice.admin.inc
 *   Administer invoices
 */

/**
 * Called through invoice_settings()
 * @ingroup form
 */
function _invoice_settings() {
  $form['ec_invoice_screens'] = array(
    '#type' => 'fieldset',
    '#title' => t('Order of Invoice Screens'),
    '#tree' => true,
    '#theme' => 'invoice_settings_order_form',
  );
  foreach (invoice_get_screens() as $key => $module) {
    $form['ec_invoice_screens'][$module] = array(
      '#type' => 'textfield',
      '#default_value' => $key,
      '#size' => 5,
      '#maxlength' => 2,
    );
  }
  $form['ec_invoice_terms'] = array(
    '#type' => 'textfield',
    '#title' => t('Invoice Terms'),
    '#size' => 60,
    '#default_value' => variable_get('ec_invoice_terms', '4 Weeks'),
    '#description' => t('Enter the terms for any invoice that is created. Values like "4 Weeks" or "Next Month" can be used. See !help_url1 or !help_url2 for more information.<br />The current value will return a due date of %due from %now',
      array('!help_url1' => l('http://php.net/strtotime', 'http://php.net/strtotime'),
      '!help_url2' => l('http://www.gnu.org/software/tar/manual/html_node/tar_109.html',
      'http://www.gnu.org/software/tar/manual/html_node/tar_109.html'),
      '%due' => format_date(strtotime(variable_get('ec_invoice_terms', '4 Weeks'))),
      '%now' => format_date(time()))),
  );
  $form['#submit'] = array(
    'invoice_settings_form_submit' => array(),
    'system_settings_form_submit' => array()
  );
  return system_settings_form($form);
}

/**
 * Called through invoice_page()
 * @return
 *   HTML, the invoice page
 */
function _invoice_page() {
  global $user;

  $header = array(
    array('data' => 'Invoice'),
    array('data' => 'Created', 'field' => 'i.created', 'sort' => 'ASC'),
    array('data' => 'Changed', 'field' => 'i.changed'),
    array('data' => 'Last Changed by', 'field' => 'u.uid'),
    array('data' => 'Operations'),
  );
  $sql = 'SELECT i.*, u.name FROM {ec_invoice} i LEFT JOIN {users} u ON i.uid = u.uid';
  $sql .= tablesort_sql($header);
  $result = pager_query($sql, 50);
  $rows = array();

  while ($data = db_fetch_object($result)) {
    $invoice = unserialize($data->invoice);
    $rows[] = array(
      array('data' => isset($invoice->address['billing']->firstname) ?
        "{$invoice->address['billing']->firstname} {$invoice->address['billing']->lastname}" :
        $invoice->invoice_id),
      array('data' => format_date($data->created)),
      array('data' => format_date($data->changed)),
      array('data' => l($data->name, "user/$data->uid")),
      array('data' => l(t('edit'), "admin/store/invoice/{$invoice->invoice_id}/edit") .
        ' '. l(t('delete'), "admin/store/invoice/{$invoice->invoice_id}/delete")),
    );
  }

  if (!empty($rows)) {
    $output .= theme('table', $header, $rows);
    $output .= theme('pager', null, 50, 0);
  }
  else {
    $output .= t('No pending invoices currently available');
  }
  return $output;
}

/**
 * Called through invoice_add()
 * @param $invoice_id
 *   Number, the invoice ID
 */
function _invoice_add($invoice_id) {
  if ($invoice = invoice_get($invoice_id)) {
    if (isset($_REQUEST['destination']) || isset($_REQUEST['edit']['destination'])) {
      $invoice->destination = isset($_REQUEST['destination']) ? $_REQUEST['destination'] :
        $_REQUEST['edit']['destination'];
      unset($_REQUEST['destination'], $_REQUEST['edit']['destination']);
    }
    else {
      unset($invoice->destination);
    }
    invoice_save($invoice_id, $invoice);
    // we already have an invoice for this contact, go and edit it.
    drupal_goto("admin/store/invoice/{$invoice_id}/edit"); // TODO: Fix destination.
  }

  $invoice = new StdClass;
  $invoice->invoice_id = $invoice_id;
  $invoice->type = 'invoice';
  if (isset($_REQUEST['destination']) || isset($_REQUEST['edit']['destination'])) {
    $invoice->destination = isset($_REQUEST['destination']) ? $_REQUEST['destination'] :
      $_REQUEST['edit']['destination'];
    unset($_REQUEST['destination'], $_REQUEST['edit']['destination']);
  }

  foreach (invoice_get_screens() as $module) {
    invoice_invoke($module, $invoice, 'create', $invoice_id);
  }

  if (!form_get_errors()) {
    invoice_save($invoice->invoice_id, $invoice);
    drupal_goto("admin/store/invoice/{$invoice_id}/edit");
  }
  else {
    drupal_goto();
  }
}

/**
 * Called through invoice_edit_existing()
 * @param $txnid
 *   Number, the transaction ID
 */
function _invoice_edit_existing($txnid = null) {
  $id = "txnid_{$txnid}";
  if ($invoice = invoice_get($id)) {
    if (isset($_REQUEST['destination']) || isset($_REQUEST['edit']['destination'])) {
      $invoice->destination = isset($_REQUEST['destination']) ? $_REQUEST['destination'] :
        $_REQUEST['edit']['destination'];
      unset($_REQUEST['destination'], $_REQUEST['edit']['destination']);
    }
    else {
      unset($invoice->destination);
    }
    invoice_save($id, $invoice);
    // we already have an invoice for this contact, go and edit it.
    drupal_goto("admin/store/invoice/{$id}/edit"); // TODO: Fix destination.
  }

  if ($invoice = store_transaction_load($txnid)) {
    $invoice->invoice_id = $id;
    if (isset($_REQUEST['destination']) || isset($_REQUEST['edit']['destination'])) {
      $invoice->destination = isset($_REQUEST['destination']) ? $_REQUEST['destination'] :
        $_REQUEST['edit']['destination'];
      unset($_REQUEST['destination'], $_REQUEST['edit']['destination']);
    }

    if (!empty($invoice->items)) {
      $items = array();
      foreach ($invoice->items as $item) {
        $items[$item->nid] = $item;
      }
      $invoice->items = $items;
    }

    invoice_save($invoice->invoice_id, $invoice);
    drupal_goto("admin/store/invoice/{$id}/edit");
  }
  else {
    drupal_not_found();
    exit();
  }
}
