<?php
/**
 * $Id: product.admin.inc,v 1.1.2.11 2007/08/23 15:57:04 sime Exp $
 * @file product.admin.inc
 * All admin processes are in a separated file to minimize the
 * amount of data loaded by every single user.
 * Only store admins will load this file!
 * @author e-Commerce Dev Team
 */

/**
 * Present a product submission form or a set of links to such forms.
 *
 * This function is adapted from node_add().
 */
function _product_add() {
  global $user;
  $edit = isset($_POST['edit']) ? $_POST['edit'] : '';
  // If a product type has been specified, validate its existence.
  $ptypes = product_get_ptypes('names');
  if (arg(3) && in_array(arg(3), array_keys($ptypes))) {
    $type = arg(3);
    // Initialize settings:
    $node = array(
      'uid'   => $user->uid,
      'name'  => $user->name,
      'type'  => 'product',
      'ptype' => $type
    );
    // Allow the following fields to be initialized via $_GET (e.g. for use
    // with a "blog it" bookmarklet):
    foreach (array('title', 'teaser', 'body') as $field) {
      if (!empty($_GET['edit'][$field])) {
        $node[$field] = $_GET['edit'][$field];
      }
    }
    $output = drupal_get_form('product_node_form', $node);
    drupal_set_title(t('Submit %name', array('%name' => $ptypes[$type])));
  }
  else {
    $output = product_types_listing();
  }
  return $output;
}

/**
 * List all product types
 */
function _product_admin_ptypes() {
  $ptypes = product_get_ptypes();
  uasort($ptypes, create_function('$a, $b', 'if ($a->name == $b->name) { return 0; } else { return strcmp($a->name, $b->name); }'));
  $output = '';
  if (!empty($ptypes)) {
    $head = array(t('Name'), t('Type'), t('Description'), array(
      'data' => t('Operations'), 'colspan' => 2));
    foreach ($ptypes as $ptype => $item) {
      $rows[] = array(
        array('data' => l($item->name, "admin/ecsettings/ptypes/{$ptype}")),
        array('data' => check_plain($ptype)),
        array('data' => check_plain($item->description)),
        array('data' => l(t('edit'), "admin/ecsettings/ptypes/{$ptype}")),
        array('data' => !isset($item->internal) ? l(t('delete'),
          "admin/ecsettings/ptypes/{$ptype}/delete") : ''),
      );
    }
    $output .= theme('table', $head, $rows);
  }
  else {
    drupal_set_message(t('No product types defined.'));
  }
  return $output;
}

/**
 * Add or Edit product type form
 */
function _product_admin_ptypes_form($info = NULL) {
  $form = array();
  if (empty($info)) {
    $info = new stdClass;
    $info->name = $info->ptype = $info->description = '';
  }
  else {
    $form['old_ptype'] = array(
      '#type' => 'value',
      '#value' => $info->ptype,
    );
  }
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#default_value' => $info->name,
    '#description' => t('The human readable name of this product type.'),
    '#required' => TRUE,
  );
  $form['ptype'] = array(
    '#type' => 'textfield',
    '#title' => t('Type'),
    '#default_value' => $info->ptype,
    '#description' => t('The machine-readable name of this product type. This is used by the system internally to refer to this product type.'),
    '#required' => TRUE,
  );
  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#default_value' => $info->description,
    '#description' => t('A brief description of what this product type is.'),
    '#required' => TRUE,
  );
  $form['info'] = array(
    '#type' => 'value',
    '#value' => $info,
  );
  $form['submit'] = array();
  $form['submit']['save'] = array(
    '#type' => 'submit',
    '#value' => t('Submit product type'),
  );
  if (isset($info->internal) && empty($info->internal)) {
    $form['submit']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset product type'),
    );
  }
  elseif (!empty($info->ptype) && !db_result(db_query_range('SELECT COUNT(*) FROM {ec_product}
      WHERE ptype = "%s"', $info->ptype, 0, 1))) {
    $form['submit']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete product type'),
    );
  }
  return $form;
}

/**
 * @param $form_id
 * @param $form_values
 * @ingroup form
 */
function _product_admin_ptypes_form_submit($form_id, $form_values) {
  $info = $form_values['info'];
  $info->name = $form_values['name'];
  $info->ptype = $form_values['ptype'];
  $info->old_ptype = isset($form_values['old_ptype']) ? $form_values['old_ptype'] : NULL;
  $info->description = $form_values['description'];
  if ($form_values['op'] == t('Reset product type')) {
    product_ptypes_delete($info->ptype);
    drupal_set_message(t('Product type %ptype has been reset',
      array('%ptype' => $info->ptype)));
    return 'admin/ecsettings/ptypes/'. $info->ptype;
  }
  elseif ($form_values['op'] == t('Delete product type')) {
    return 'admin/ecsettings/ptypes/'. $info->ptype .'/delete';
  }
  elseif ($form_values['op'] == t('Submit product type')) {
    product_ptypes_save($info);
  }
  return 'admin/ecsettings/ptypes';
}

/**
 * @param $form_id
 * @param $form_values
 * @ingroup form
 */
function _product_admin_ptypes_form_validate($form_id, $form_values) {
  if (!empty($form_values['old_ptype']) && $form_values['ptype'] != $form_values['old_ptype'] &&
      product_get_ptypes('type', $form_values['ptype'])) {
    form_set_error('ptype', t('Product type %ptype already exists',
      array('%ptype' => $form_values['ptype'])));
  }
}

/**
 * Implementation of hook_delete().
 */
function _product_delete($node, $revision_only = FALSE) {
  if (!empty($revision_only)) {
    db_query('DELETE FROM {ec_product} WHERE nid = %d AND vid = %d', $node->nid, $node->vid);
  }
  else {
    db_query('DELETE FROM {ec_product} WHERE nid = %d', $node->nid);
  }
  module_invoke('cart', 'productapi', $node, 'delete');
  product_invoke_productapi($node, 'delete', $revision_only);
  product_invoke_all_features($node, 'delete', $revision_only);
  drupal_set_message(t('Product deleted'));
}

/**
 * @param $form_id
 * @param $form_values
 * @ingroup form
 */
function _product_edit_form(&$invoice) {
  $form['#theme'] = 'product_edit_form';
  $form['products'] = array(
    '#type' => 'fieldset',
    '#title' => t('Products'),
    '#tree' => TRUE,
    '#theme' => 'product_edit_form_product',
  );
  foreach ((array)$invoice->items as $key => $item) {
    $node = node_load($item->nid);
    $form['products'][$node->nid]['#node'] = $item;
    $form['products'][$node->nid]['title'] = array(
      '#value' => l($item->title, 'node/'. ($node->pparent ? $node->pparent : $node->nid)),
    );
    $price = store_adjust_misc($invoice, $item);
    $form['products'][$node->nid]['#total'] = product_has_quantity($node) ?
      $price * $item->qty : $price;
    if (!empty($node->is_recurring)) {
      $form['products'][$node->nid]['recurring'] = array(
        '#value' => product_recurring_nice_string($node)
      );
    }
    if ($node->ptype == 'tangible' && $node->availability != NULL && $node->availability != 1) {
      $form['products'][$node->nid]['availability'] = array(
        '#value' => availability_get_message($node->availability)
      );
    }
    $form['products'][$node->nid]['qty'] = array(
      '#type' => product_has_quantity($node) ? 'textfield' : 'value',
      '#default_value' => $item->qty,
      '#size' => 2,
      '#maxlength' => 2
    );
    $form['products'][$node->nid]['delete'] = array(
      '#type' => 'checkbox',
    );
    if ($extra = product_invoke_productapi($item, 'cart_form', $invoice)) {
      $form['#product_cart_form'] = TRUE;
      $form['products'][$node->nid]['data'] = array();
    }
  }
  $form['add'] = array(
    '#type' => 'fieldset',
    '#title' => t('Add Products'),
    '#collapsible' => TRUE,
    '#collapsed' => count($invoice->items) ? TRUE : FALSE,
  );
  $form['add']['new_products'] = array(
    '#type' => 'textfield',
    '#title' => t('Product Ids'),
    '#maxlength' => 60,
    '#autocomplete_path' => 'product/autocomplete/all',
    '#description' => t('Enter the id\'s of the products that you which to add to the invoice. Here is a <a href="!product_quicklist" onclick="window.open(this.href, \'!product_quicklist\', \'width=480,height=480,scrollbars=yes,status=yes\'); return FALSE">list of all products</a>.', array('!product_quicklist' => url('admin/store/products/quicklist'))),
  );
  $form['submit']['invoiceop'] = array(
    '#type' => 'submit',
    '#value' => t('Update Products'),
    '#attributes' => array('id' => 'invoiceop'),
  );
  $form['submit']['continue'] = array(
    '#type' => 'submit',
    '#value' => t('Continue'),
  );
  return $form;
}

/**
 * Implementation of hook_form().
 */
function _product_form(&$node) {
  $node_type = node_get_types('type', $node);
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => !empty($node_type->title_label) ? $node_type->title_label : t('Name'),
    '#required' => TRUE,
    '#default_value' => !empty($node->title) ? $node->title : '',
    '#weight' => -5
  );
  $form['body_filter']['body'] = array(
    '#type' => 'textarea',
    '#title' => !empty($node_type->body_label) ? $node_type->body_label : t('Description'),
    '#default_value' => !empty($node->body) ? $node->body : '',
    '#description' => t('Provide the product details to display on the page.'),
    '#rows' => 20
  );
  $form['body_filter']['format'] = !empty($node->format) ? filter_form($node->format) : filter_form(NULL);
  // Get base form elements.
  $form['product'] = _product_form_base_elements($node);
  // Get the product-type-specific bits.
  $form['product'] = array_merge($form['product'], (array) product_invoke_productapi($node, 'form'));
  return $form;
}

/**
 * Elements common to every single product: price, sku and
 * the option to show a "Add to cart" as a link or a button
 */
function _product_form_base_elements(&$node) {
  $form['ptype'] = array(
    '#type' => 'value',
    '#value' => $node->ptype
  );
  $form['price'] = array(
    '#type' => 'price',
    '#title' => t('Price'),
    '#size' => 25,
    '#maxlength' => 50,
    '#required' => TRUE,
    '#default_value' => format_currency(!empty($node->price) ? $node->price : 0),
    '#description' => t('How much does this product retail for? Note: This price may be different from the selling price due to price adjustments elsewhere.'),
    '#weight' => -20,
  );
  $form['sku'] = array(
    '#type' => 'textfield',
    '#title' => t('SKU'),
    '#size' => 25,
    '#maxlength' => 50,
    '#default_value' => !empty($node->sku) ? $node->sku : '',
    '#description' => t('If you have an unique identifier for this product from another system or database, enter that here. This is optional, as system IDs are automatically created for each product.'),
    '#weight' => -10,
  );
  $form['hide_cart_link'] = array(
    '#type' => 'radios',
    '#title' => t('"Add to cart" link'),
    '#default_value' => isset($node->hide_cart_link) ? $node->hide_cart_link : 0,
    '#options' => array(t('Visible'), t('Hidden')),
    '#description' =>  t('Choose whether or not you want the "Add to cart" link visible for this product.'),
    '#weight' => -5,
  );
  return $form;
}

/**
 * Generates product type options fieldset.
 */
function _product_form_transform(&$node) {
  drupal_add_css(drupal_get_path('module', 'product') .'/product.css');
  // Product type collapsed fieldset
  $form = array(
    '#type' => 'fieldset',
    '#title' => t('Product'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );
  // Choose a product type
  $ptypes[FALSE] = array('name' => t('Not a product'), 'help' => t('This item not for sale.'));
  $types = variable_get('ec_product_ptypes_'. $node->type, array());
  if (empty($types)) {
    $types = array_keys(product_get_ptypes('names'));
  }
  // Populate product type properties.
  foreach ($types as $key) {
    $info = product_get_ptypes('type', $key);
    if (product_ptypes_access($info->name)) {
      $ptypes[$key] = array(
        'name' => $info->name,
        'help' => $info->description,
      );
    }
  }
  // Display each radio option, with descriptive text
  foreach ($ptypes as $key => $data) {
    $form['ptype'][$key] = array(
      '#type' => 'radio',
      '#title' => $data['name'],
      '#return_value' => $key,
      '#default_value' => $node->ptype,
      '#description' => $data['help'],
      '#parents' => array('ptype'),
     );
  }
  $form['product_add'] = array(
    '#type' => 'button',
    '#value' => t('Add to store')
  );
  return $form;
}

/**
 * Generates product form fields to be added into the node form.
 * @param &$node
 * @ingroup form
 */
function _product_form_transform_product(&$node) {
  // Get base form elements.
  $form = product_form_base_elements($node);
  // Change ptype from 'value' to 'hidden'
  $form['ptype']['#type'] = 'hidden';
  // Get the product-type-specific bits.
  $form = array_merge($form, (array) product_invoke_productapi($node, 'form'));
  if (variable_get('ec_product_convert_'. $node->type, 0) != 1) {
    $form['product_remove'] = array(
      '#type' => 'checkbox',
      '#title' => t('Remove from store'),
      '#description' => t('Check here to delete product information.  Takes effect when changes are submitted.')
    );
  }
  return $form;
}

/**
 * Display all products in a table format.
 */
function _product_overview() {
  $header = array(
    array('data' => t('nid'), 'field' => 'p.nid', 'sort' => 'asc'),
    array('data' => t('sku'), 'field' => 'p.sku'),
    array('data' => t('title'), 'field' => 'n.title'),
    array('data' => t('price'), 'field' => 'p.price'),
    array('data' => t('type'), 'field' => 'p.ptype'),
  );
  $sql = 'SELECT n.nid, n.title, p.* FROM {node} n
    INNER JOIN {ec_product} p ON n.vid = p.vid
    WHERE n.status = 1'. tablesort_sql($header);
  $result = pager_query(db_rewrite_sql($sql), 50);
  while ($data = db_fetch_object($result)) {
    $rows[] = array(l($data->nid, "node/$data->nid/edit"), $data->sku,
      l($data->title, "node/$data->nid"), format_currency(product_adjust_price($data, 'product') +
      product_get_specials($data, 'product', TRUE)), $data->ptype);
  }
  $pager = theme('pager', NULL, 50, 0);
  if (!empty($pager)) {
    $rows[] = array(array('data' => $pager, 'colspan' => 6));
  }
  return theme('product_overview', $rows, $header);
}

/**
 * Generate a hardcoded product list. Uses echo to
 * directly print the result.
 */
function _product_quicklist() {
  $html = store_quicklist_header();
  $header = array(
    array('data' => t('nid'), 'field' => 'p.nid', 'sort' => 'asc'),
    array('data' => t('sku'), 'field' => 'p.sku'),
    array('data' => t('title'), 'field' => 'n.title'),
    array('data' => t('price'), 'field' => 'p.price'),
    array('data' => t('type'), 'field' => 'p.ptype')
  );
  $sql = 'SELECT * FROM {node} n, {ec_product} p WHERE n.vid = p.vid AND n.status = 1'. tablesort_sql($header);
  $result = pager_query(db_rewrite_sql($sql), 50);
  while ($data = db_fetch_object($result)) {
    $rows[] = array($data->nid, $data->sku, $data->title,
      format_currency(product_adjust_price($data, 'product') +
      product_get_specials($data, 'product', TRUE)), $data->ptype);
  }
  $pager = theme('pager', NULL, 50, 0);
  if (!empty($pager)) {
    $rows[] = array(array('data' => $pager, 'colspan' => 5));
  }
  echo $html;
  if (!empty($rows)) {
    echo theme('table', $header, $rows);
  }
  else {
    echo t('<p>There are no products to view. You can <a href="!add_product">add a product</a>.</p>',
      array('!add_product' => 'node/add/product'));
  }
  echo '</body></html>';
}

/**
 * Assumes the node data has already been saved to its respective tables and
 * only updates the main product table and the plugin of the product type
 * (e.g., product_tangible).
 */
function _product_save($node) {
  if (!empty($node->ptype)) {
    $fields = product_fields();
    /* Be very sure that we have a product entry to update! */
    if (db_result(db_query('SELECT COUNT(vid) FROM {ec_product} WHERE vid = %d', $node->vid))) {
      foreach ($fields as $field) {
        if (isset($node->$field)) {
          $q[] = "{$field} = '%s'";
          $v[] = $node->$field;
        }
      }
      $v[] = $node->vid;
      db_query('UPDATE {ec_product} SET '. implode(', ', $q) .' WHERE vid = %d', $v);
      product_invoke_productapi($node, 'update');
      product_invoke_all_features($node, 'update');
    }
    else {
      foreach ($fields as $field) {
        if (isset($node->$field)) {
          $k[] = $field;
          $v[] = $node->$field;
          $s[] = '"%s"';
        }
      }
      db_query('INSERT INTO {ec_product} ('. implode(', ', $k) .') VALUES('. implode(', ', $s) .')', $v);
      product_invoke_productapi($node, 'insert');
      product_invoke_all_features($node, 'insert');
    }
  }
}

function _product_admin_ptypes_feature_list($info) {
  $form = array();
  if (!empty($info->features)) {
    $form['features'] = array(
      '#tree' => TRUE,
    );
    foreach ($info->features as $ftype => $feature) {
      $form['features'][$ftype]['ptype'] = array(
        '#type' => 'value',
        '#value' => $info->ptype,
      );
      $form['features'][$ftype]['ftype'] = array(
        '#type' => 'value',
        '#value' => $feature->ftype,
      );
      $form['features'][$ftype]['name'] = array(
        '#value' => $feature->name,
      );
      $form['features'][$ftype]['description'] = array(
        '#value' => $feature->description,
      );
      $form['features'][$ftype]['weight'] = array(
        '#type' => 'weight',
        '#default_value' => $feature->weight,
        '#disabled' => isset($feature->allow_edit) ? $feature->allow_edit : FALSE,
      );
      if (!isset($feature->allow_disable) || $feature->allow_disable) {
        $form['features'][$ftype]['delete'] = array(
          '#value' => l(t('delete'), 'admin/ecsettings/ptypes/'. $info->ptype .
          '/features/'. $feature->ftype .'/delete'),
        );
      }
    }
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => 'Update features',
    );
  }
  else {
    drupal_set_message(t('No features have been added to this product'));
  }
  return $form;
}

/**
 * @param $form_id
 * @param $form_values
 * @ingroup form
 */
function product_admin_ptypes_feature_list_submit($form_id, $form_values) {
  foreach ($form_values['features'] as $feature) {
    product_feature_update((object)$feature);
  }
  drupal_set_message(t('Features have been updated'));
}

/**
 * @ingroup themeable
 */
function theme_product_admin_ptypes_feature_list($form) {
  $output = '';

  if (isset($form['features'])) {
    $heading = array(t('Name'), t('Description'), t('Weight'), t('Operations'));
    foreach (element_children($form['features']) as $element) {
      $rows[] = array(
        array('data' => drupal_render($form['features'][$element]['name'])),
        array('data' => drupal_render($form['features'][$element]['description'])),
        array('data' => drupal_render($form['features'][$element]['weight'])),
        array('data' => isset($form['features'][$element]['delete']) ?
          drupal_render($form['features'][$element]['delete']) : ''),
      );
    }

    $output .= theme('table', $heading, $rows);
  }

  return $output . drupal_render($form);
}

/**
 * Link product types to features
 * @param $info
 */
function _product_admin_ptypes_feature_form($info) {
  $options = $form = array();
  $features = product_get_features();
  foreach ($features as $ftype => $feature) {
    if (!isset($info->features[$ftype])) {
      $options[$ftype] = t('@name<dt>@description</dt>',
        array('@name' => $feature->name, '@description' => $feature->description));
    }
  }
  if (!empty($options)) {
    $form['info'] = array(
      '#type' => 'value',
      '#value' => $info,
    );
    $form['ftype'] = array(
      '#type' => 'radios',
      '#title' => t('Select a product feature to be added'),
      '#options' => $options,
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Add product feature'),
    );
  }
  else {
    drupal_set_message(t('No more features can be added to @name',
      array('@name' => $info->name)));
    drupal_goto('admin/ecsettings/ptypes/'. $info->ptype .'/features');
  }
  return $form;
}

/**
 * @param $form_id
 * @param $form_values
 * @ingroup form
 */
function product_admin_ptypes_feature_form_submit($form_id, $form_values) {
  product_feature_enable($form_values['info'], $form_values['ftype']);
  return 'admin/ecsettings/ptypes/'. $form_values['info']->ptype .'/features';
}

/**
 * @ingroup themeable
 */
function theme_product_edit_form(&$form) {
  $output .= drupal_render($form['products']);
  $output .= drupal_render($form);
  return $output;
}

/**
 * @ingroup themeable
 */
function theme_product_edit_form_product(&$form) {
  $children = element_children($form);
  if (!empty($children)) {
    $extra = FALSE;

    foreach ($children as $nid) {
      if (!empty($form[$nid]['data'])) {
        $extra = TRUE;
        break;
      }
    }

    foreach ($children as $nid) {
      $total+= $form[$nid]['#total'];
      $desc = drupal_render($form[$nid]['title']) .'<br />';
      if (!empty($form[$nid]['recurring'])) {
        $desc .= '<div class="recurring-details">'. drupal_render($form[$nid]['recurring']) .'</div>';
      }
      if (!empty($form[$nid]['availability'])) {
        $desc .= drupal_render($form[$nid]['availability']);
      }
      $desc .= '<p>'. format_currency($form[$nid]['#total']) .'</p>';
      $row = array(
        array('data' => $desc),
        array('data' => drupal_render($form[$nid]['qty'])),
      );
      if (!empty($extra)) {
        if (!empty($form[$nid]['node'])) {
          $row[] = array('data' => drupal_render($form[$nid]['node']));
          $extra = TRUE;
        }
        else {
          $row[] = '';
        }
      }
      $row[] = array('data' => drupal_render($form[$nid]['delete']), 'align' => 'center');
      $rows[] = $row;
    }
    $header = array(
      array('data' => t('Items')),
      array('data' => t('Qty')),
    );
    if (!empty($extra)) {
      $header[] = array('data' => '');
    }
    $header[] = array('data' => t('Delete'));
    $output .= theme('table', $header, $rows);
  }
  else {
    $output .= '<p>'. t('No Products Currently on this Invoice') .'</p>';
  }
  $output .= drupal_render($form);
  return $output;
}

/**
 * Display all products in a table format.
 * @ingroup themable
 */
function theme_product_overview($rows, $header) {
  if (!empty($rows)) {
    $output = theme('table', $header, $rows);
  }
  else {
    $output = '<p>'. t('There are no products to view.') .'</p>';
  }
  return $output;
}
