<?php
/**
 * $Id: product.views.inc,v 1.1.2.4 2007/08/13 19:39:56 brmassa Exp $
 * @author e-Commerce Dev Team
 * @file product.views.inc
 * Product integration with Views.
 */

/**
 * Set a standard views. The /products page will show a list
 * of available products, similar to old hardcoded products page.
 */
function product_views_default_views() {
  $view = new stdClass();
  $view->name               = 'Products';
  $view->description        = 'Product listing';
  $view->access             = array();
  $view->view_args_php      = '';
  $view->page               = true;
  $view->page_title         = 'products';
  $view->page_header        = '';
  $view->page_header_format = '1';
  $view->page_footer        = '';
  $view->page_footer_format = '1';
  $view->page_empty         = '';
  $view->page_empty_format  = '1';
  $view->page_type          = 'product_list';
  $view->url                = 'product';
  $view->use_pager          = true;
  $view->nodes_per_page     = '9';
  $view->menu               = true;
  $view->menu_title         = 'Products';
  $view->sort               = array();
  $view->argument           = array();
  $view->field              = array(
    array(
      'tablename' => 'node',
      'field' => 'title',
      'label' => '',
      'handler' => 'views_handler_field_nodelink',
      'options' => 'link',
    ),
    array(
      'tablename' => 'node',
      'field' => 'body',
      'label' => '',
      'handler' => 'views_handler_field_teaser',
    )
  );
  $view->filter             = array(
    array(
      'tablename' => 'node',
      'field' => 'status',
      'operator' => '=',
      'options' => '',
      'value' => '1',
    ),
    array(
      'tablename' => 'node',
      'field' => 'type',
      'operator' => 'OR',
      'options' => '',
      'value' => array('product'),
    )
  );
  $view->exposed_filter     = array();
  $view->requires           = array(node, ec_product);
  $views[$view->name] = $view;
  return $views;
}

/**
 * Field handler to give an add to cart link using product_link()
 */
function product_views_handler_add_to_cart_link($fieldinfo, $fielddata, $value, $data) {
  $node = node_load($data->nid);
  $links = module_invoke('product', 'link', 'node', $node);
  foreach (module_implements('link_alter') as $module) {
    $function = $module .'_link_alter';
    $function($node, $links);
  }
  $link = array_pop($links);
  $html = isset($link['html']) && $link['html'];
  if (!empty($link['href'])) {
    return l($link['title'], $link['href'], $link['attributes'], $link['query'], $link['fragment'], false, $html);
  }
  elseif (!empty($link['title'])) {
    //Some links are actually not links, but we wrap these in <span> for adding title and class attributes
    if (empty($html)) {
      $link['title'] = check_plain($link['title']);
    }
    return '<span'. drupal_attributes($link['attributes']) .'>'. $link['title'] .'</span>';
  }
}

/**
 * Set a new listing style tailored for products
 * @return
 *   Array, new listing styles
 */
function product_views_style_plugins() {
  $items['product_list'] = array(
    'name' => t('Product: List of products'),
    'theme' => 'product_views_product_list',
    'summary_theme' => 'views_summary',
  );
  return $items;
}

/**
 * Implementation of hook_views_tables()
 */
function product_views_tables() {
  $tables['ec_product'] = array(
    'name' => 'ec_product',
    'join' => array(
      'left' => array(
        'table'     => 'node',
        'field'     => 'vid'
      ),
      'right' => array(
        'field'     => 'vid'
      )
    ),
    'fields' => array(
      'sku' => array(
        'name'      => t('Product: SKU'),
        'sortable'  => true,
      ),
      'price' => array(
        'name' => t('Product: Price'),
        'sortable'  => true,
      ),
      'addtocartlink' => array(
        'handler'   => 'product_views_handler_add_to_cart_link',
        'name'      => t('Product: Add to cart link'),
        'sortable'  => false,
        'notafield' => true,
      )
    ),
    'sorts' => array(
      'sku' => array(
        'name' => t('Product: SKU'),
      ),
      'price' => array(
        'name' => t('Product: Price'),
      )
    ),
    'filters' => array(
      'ptype' => array(
        'help'      => 'Filter by product type',
        'name'      => 'Product: Type',
        'operator'  => 'views_handler_operator_andor',
        'value'     => array(
          '#multiple' => true,
          '#options'  => product_get_ptypes('names'),
          '#type'     => 'select'
        ),
        'value-type' => 'array'
      )
    )
  );
  return $tables;
}

/**
 * Themes a product page into a table similar to that in the previous 4.7 product page.
 * @ingroup themeable
 */
function theme_product_views_product_list($view, $nodes, $type) {
  $teasers = true;
  $links = true;
  $rows = $row = array();
  foreach ($nodes as $count => $n) {
    $node = node_load($n->nid);
    if (!($count % 3)) {
      $rows[] = $row;
      $row = array();
    }
    array_push($row, node_view($node, $teasers, false, $links));
  }
  if (!empty($row)) {
    array_push($rows, $row);
  }
  drupal_add_css(drupal_get_path('module', 'cart') .'/cart.css');
  return theme('table', array(), $rows, array('class' => 'product-table'));
}
