<?php
// $Id: ec_query.inc,v 1.3.2.9 2007/07/29 22:25:04 brmassa Exp $

/**
 * e-Commerce query object
 */
class _ec_query {

  var $fields = array('st.txnid', 'st.*');
  var $tables = array(
    'st' => array('table' => 'ec_transaction', 'alias' => 'st'),
  );
  var $where = array();
  var $group = array();
  var $having = array();
  var $order = array();
  var $distinct = false;

  function _ec_query() {
  }

  function add_select($field) {
    $this->fields[] = $field;
  }

  function ensure_table($table, $alias, $type = null, $join = null) {
    if (!isset($this->tables[$alias])) {
      $this->tables[$alias] = array('table' => $table,
        'alias' => $alias,
        'type' => $type,
        'join' => $join
      );
    }
  }

  function add_where($where) {
    $this->where[] = $where;
  }

  function add_group($group) {
    $this->group[] = $group;
  }

  function add_having($having) {
    $this->having[] = $having;
  }

  function add_order($order) {
    $order = trim($order);
    if (!empty($order)) {
      $this->order[] = $order;
    }
  }

  function is_distinct() {
    $this->distinct = true;
  }

  function query($count = false) {
    $this->_prepare_query();

    $sql = 'SELECT ';
    if (!empty($count)) {
      $sql .= 'COUNT('. ($this->distinct ? 'DISTINCT ' : '') ." {$this->fields[0]})";
    }
    else {
      $sql .= ($this->distinct ? 'DISTINCT ' : '') . implode(',', $this->fields);
    }
    $sql .= ' FROM '. implode(' ', $this->from);
    if (!empty($this->where)) {
      $sql .= ' WHERE '. implode(' AND ', $this->where);
    }
    if (!empty($this->group) && empty($count)) {
      $sql .= ' GROUP BY '. implode(',', $this->group);
    }
    if (!empty($this->having)) {
      $sql .= ' HAVING '. implode(' AND ', $this->having);
    }
    if (!empty($this->order) AND empty($count)) {
      $sql .= ' ORDER BY '. implode(',', $this->order);
    }
    return $sql;
  }

  function get_tables() {
    foreach ($this->tables as $table) {
      $tables[] = $table['table'];
    }
    return array_unique($tables);
  }

  function _prepare_query() {
    $this->fields = array_unique($this->fields);
    $this->where = array_unique($this->where);
    $this->group = array_unique($this->group);
    $this->order = array_unique($this->order);

    $this->from = array();
    foreach ($this->tables as $table) {
      $from = !empty($table['type']) ? "{$table['type']} JOIN " : '';
      $from .= "{{$table['table']}} {$table['alias']}";
      $from .= !empty($table['join']) ? " ON {$table['join']}" : '';
      $this->from[] = $from;
    }
  }
}
