<?php
// $Id: store.admin.inc,v 1.1.2.27 2007/09/10 05:32:37 brmassa Exp $

/**
 * Called by store_cron()
 */
function _store_cron() {
  /* Any item whose workflow is 'shipped', move it to 'completed'. This does
     seem presumptous. Ideally we'd email the user a list of shipped items for
     day and then adjust the shipping workflow after a given time period, such
     as 2 week (in case of returns or something) rather than automatically
     closing it here.  */
  db_query('UPDATE {ec_transaction} SET workflow = %d WHERE workflow = %d AND allocation = %d', EC_WORKFLOW_COMPLETED, EC_WORKFLOW_SHIPPED, EC_ALLOC_COMPLETED);

//   if (variable_get('payment_notices', 0)) {
  if (variable_get('payment_notices', 0) and
      (time() >= (variable_get('ecommerce_store_summary_email_sent', 0) +
      variable_get('ecommerce_store_summary_email_frequency', 60 * 60 * 24)))) {
    global $base_url;

    // Find order(s) with a 'cleared' payment status needs action.
    if ($count = db_result(db_query('SELECT COUNT(st.txnid) FROM {ec_transaction} AS st WHERE st.allocation = %d AND (st.workflow = %d OR st.workflow = %d OR st.workflow = %d OR st.workflow = %d)', EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED, EC_WORKFLOW_INVOICED, EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE))) {

      $beg_month    = mktime(0, 0, 0, date('m'), 1, date('Y'));
      $mon_orders  = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction} where created >= %d AND allocation = %d AND workflow = %d', $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED));
      $mon_sales    = '$'. number_format(db_result(db_query('SELECT SUM(gross) FROM {ec_transaction} where created >= %d AND allocation = %d AND workflow = %d', $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED)), 2);

      $buf = t('%count order(s) with a "cleared" payment status needs action:',
        array('%count' => $count)) ."\n";
      $result = db_query('SELECT * FROM {ec_transaction} AS st WHERE st.allocation = %d AND st.workflow IN (%d, %d, %d, %d) ORDER BY st.txnid DESC', EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED, EC_WORKFLOW_INVOICED, EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE);
      while ($data = db_fetch_object($result)) {

        $buf .= t('Order #@txnid created @timestamp ago', array('@txnid' => $data->txnid,
          '@timestamp' => format_interval(time() - $data->created, 2))) ."\n";
        $buf .= t('Workflow status: @transaction-workflow',
          array('@transaction-workflow' => transaction_get_workflow($data->workflow))) ."\n";
        $buf .= t('Email: @email', array('@email' => $data->mail)) ."\n";
        $buf .= t('Total: @gross', array('@gross' => format_currency($data->gross))) ."\n";

        $r = db_query('SELECT stp.*, st.allocation, st.workflow FROM {ec_transaction} AS st, {ec_transaction_product} AS stp WHERE st.txnid = %d AND st.txnid = stp.txnid ORDER BY st.txnid DESC', $data->txnid);
        $items = null;
        $has_shippable_item = false;
        while ($item = db_fetch_object($r)) {
          if ($product = product_load($item)) {
            foreach ($product as $key => $value) {
              if ($key != 'title' && $key != 'price') {
                $item->$key = $value;
              }
            }
          }
          if (product_is_shippable($item->vid)) {
            $has_shippable_item = true;
          }

          $sku = ($item->sku) ? " [$item->sku]" : "";
          $items .= t('@order of @title at @price each', array(
            '@order' => format_plural($item->qty, '1 order', '@count orders'),
            '@title' => $item->title . $sku,
            '@price' => format_currency($item->price)));
        }

        /* If the transaction has no shippable items and the payment status is
           'complete' and the workflow is 'transaction received', set the
           workflow to 'complete' */
        if (empty($has_shippable_item) && $data->allocation == EC_ALLOC_COMPLETED && $data->workflow == EC_WORKFLOW_RECEIVED) {
          db_query('UPDATE {ec_transaction} SET workflow = %d WHERE txnid = %d', EC_WORKFLOW_COMPLETED, $data->txnid);
        }

        $shipping_to = theme('store_format_address', $data, 'shipping', 'text');
        $billing_to = theme('store_format_address', $data, 'billing', 'text');

        $buf .= t("Shipping to:\n!shipping_address", array('!shipping_address' => $shipping_to)) ."\n";
        $buf .= t("Billing to:\n!billing_address", array('!billing_address' => $billing_to)) ."\n";
        $buf .= "$items\n";
        $buf .= t('Update:') ."$base_url/admin/store/transaction/edit/$data->txnid" ."\n";
        $buf .= "=====\n";

      }

      $output = $buf;
      $output .= t('(These orders will no longer be listed once the transaction workflow is set to either "Completed" or "Canceled")') ."\n";
      $output .= t("Monthly Summary\nNumber of completed orders: @monthly-orders\nSales: @monthly-sales",
        array('@monthly-orders' => $mon_orders, '@monthly-sales' => $mon_sales)) ."\n";
      $output .= t('Want more reports? Visit @base_url/admin/store', array('@base_url' => $base_url)) ."\n";

      $subject    = t('@site-name transaction summary', array('@site-name' => variable_get('site_name', 'Drupal')));
      $to         = variable_get('site_mail', ini_get('sendmail_from'));
      $from       = variable_get('site_mail', ini_get('sendmail_from'));
      $headers    = array(
        'From'        => $from,
        'Reply-to'    => $from,
        'X-Mailer'    => 'PHP',
        'Return-path' => $from,
        'Errors-to'   => $from,
      );

      drupal_mail('ec_order_summary', $to, $subject, $output, null, $headers);
      variable_set('ecommerce_store_summary_email_sent', time());
    }
  } // End receive payment notices.
}

/**
 * Called by store_ec_settings
 */
function _store_ec_settings() {
  $site_mail = variable_get('site_mail', ini_get('sendmail_from'));
  if (empty($site_mail)) {
    $site_mail = 'no e-mail address found';
  }

  $form['ec_order_overview'] = array(
    '#type'           => 'textarea',
    '#title'          => t('Order history overview'),
    '#default_value'  => variable_get('ec_order_overview', ''),
    '#cols'           => 70,
    '#rows'           => 5,
    '#description'    => t('This text will be displayed at the top of users\' order history pages.'),
  );

  $form['product_cart_addition_by_link'] = array(
    '#type' => 'checkbox',
    '#title' => t('Users adds product to cart using link.'),
    '#default_value' => variable_get('product_cart_addition_by_link', 1),
    '#description' => t('If this field is unchecked a form will be placed at the bottom of the product view which will allow the user select the quantity and other information.'),
  );
  $form['ec_goto_cart_change'] = array(
    '#type' => 'textfield',
    '#title'          => t(''),
    '#default_value'  => variable_get('ec_goto_cart_change', 'cart/view'),
    '#description'    => t('Enter the user destination after adding a product to the cart (eg. "node/1"). If left blank, the user will remain on the same page and a message will alert them that the item has been added to their cart. To send the user to cart, enter "cart/view".'),
  );

  $form['ec_notices'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('Transaction notices'),
    '#collapsible'    => true,
    '#collapsed'      => true,
    '#description'    => t('These settings control whether the site administrator will receive email notifications of transaction reports, and the frequency thereof.'),
  );
  $form['ec_notices']['payment_notices'] = array(
    '#type'           => 'radios',
    '#title'          => t('Transaction e-mail notices'),
    '#default_value'  => variable_get('payment_notices', 0),
    '#options'        => array(t('Disabled'), t('Enabled')),
    '#description'    => t('Enable or disable transaction notifications. If enabled, the site administrator (%site_mail) will receive an email summarizing the status of all orders. The frequency of emails is dependent on how often the site administrator has scheduled to run cron jobs.  Finally, The site administrator e-mail address can be changed in the <a href="!general_config_screen">general configuration screen</a>,',
      array('%site_mail' => $site_mail,
      '!general_config_screen' => url('admin/settings')))
  );
  $form['ec_notices']['ecommerce_store_summary_email_frequency'] = array(
    '#type'          => 'select',
    '#title'         => t('Frequency of notices'),
    '#options'       => array(
      0           => t('every cron'),
      60*60       => t('1 hour'),
      60*60*12    => t('12 hours'),
      60*60*24    => t('1 day'),
      60*60*24*5  => t('5 day'),
      60*60*24*7  => t('1 week')
    ),
    '#default_value' => variable_get('ecommerce_store_summary_email_frequency', 60*60*24),
    '#description'   => t('As long as cron is running, this ensures a transaction report is sent no more than once in this period. Thus, even if cron runs every hour, you will still only receive once transaction notification e-mail.'),
  );

  // Begin email notification settings
  $form['email_notification'] = array(
    '#type' => 'fieldset',
    '#title' => t('Email Notifications'),
    '#collapsible' => true,
    '#collapsed' => true
  );
  $form['email_notification']['customer_invoice'] = ec_mail_selection_form_for_variable('store_settings_form',
    ECMAIL_TYPE_CUSTOMER_INVOICE, t('Customer invoice e-mail'), MAILVAR_CUSTOMER_INVOICE,
    t('This text will be emailed to customers after they make a purchase.'));
  $form['email_notification']['processing_error'] = ec_mail_selection_form_for_variable('store_settings_form',
    ECMAIL_TYPE_PROCESSING_ERROR, t('Payment error e-mail'), MAILVAR_PROCESSING_ERROR,
    t('This text will be e-mailed to customers if there is a payment processing problem.'));
  $form['email_notification']['ask_customer_template'] = ec_mail_selection_form_for_variable('store_settings_form',
    ECMAIL_TYPE_ASK_CUSTOMER, t('Ask customer e-mail template'), MAILVAR_ASK_CUSTOMER,
    t('This text is used as a template for asking customer.'));
  $form['email_notification']['cancel_transaction'] = ec_mail_selection_form_for_variable('store_settings_form',
    ECMAIL_TYPE_CANCEL_TXN, t('Cancel transaction e-mail'), MAILVAR_CANCEL_TXN,
    t('This text will be e-mailed to customers if their purchase has been canceled.'));

  return system_settings_form($form);
}

/**
 * Called by theme_store_invoice()
 */
function _theme_store_invoice($txn, $print_mode = true, $trial = false) {
  global $base_url;

  $header = array();
  $row    = array();

  if (empty($txn->mail) && $txn->uid > 0) {
    $txn->mail = db_result(db_query('SELECT mail FROM {users} WHERE uid = %d', $txn->uid));
  }

  if (!empty($txn->items)) {
    $header = array(t('Quantity'), t('Item'), t('Price'));

    $shippable = false;
    foreach ($txn->items as $p) {
      $prod = product_load($p);
      if (product_is_shippable($p->vid)) {
        $shippable = true;
      }
      $price = store_adjust_misc($txn, $p);

      $subtotal += (product_has_quantity($p) ? $p->qty * $price : $price);
      $details = '';
      if (0 && is_array($p->data)) {
        foreach ($p->data as $key => $value) {
          if (!empty($value)) {
            $items[] = '<strong>'. check_plain($key) .': </strong>'. check_plain($value);
          }
        }
        if (!empty($items)) {
          $details = theme('item_list', $items);
        }
      }

      $row[] = array(array('data' => $p->qty, 'align' => 'center', 'valign' => 'top'), '<em>'. check_plain($p->title) .'</em> '. (($prod->sku != '') ? '['. check_plain($prod->sku) .']' : '') .'<br />'. $details, array('data' => format_currency($price), 'valign' => 'top', 'align' => 'right'));
    }

    if (is_array($txn->misc)) {
      foreach ($txn->misc as $misc) {
        if (empty($misc->seen)) {
          $price = $misc->qty ? $misc->price * $misc->qty : $misc->price;
          $row[] = array(array('data' => "<strong>{$misc->description}</strong>: ". format_currency($price), 'colspan' => 3, 'align' => 'right'));
        }
      }
    }

    $row[] = array(array('data' => '<hr size="1" noshade="noshade" />', 'colspan' => 3, 'align' => 'right'));
    $row[] = array(array('data' => t('<strong>Total:</strong> %total', array('%total' => format_currency(store_transaction_calc_gross($txn)))), 'colspan' => 3, 'align' => 'right'));
  }

  $payment_info  = '<div>'. t('<strong>Ordered On:</strong> %order-date', array('%order-date' => format_date($txn->created))) .'</div>';
  if (!empty($txn->duedate)) {
    $payment_info .= '<div>'. t('<strong>Due Date:</strong> %due-date', array('%due-date' => format_date($txn->duedate))) .'</div>';
  }
  $payment_info .= '<div>'. t('<strong>Transaction ID:</strong> %txnid', array('%txnid' => $trial ? t('Trial Invoice - Not Yet Posted') : $txn->txnid)) .'</div>';

  $css        = base_path() . drupal_get_path('module', 'store') .'/invoice.css';
  $site_name  = t('%site-name Invoice', array('%site-name' => variable_get('site_name', 'drupal')));

  if ($shipping_to = theme('store_format_address', $txn, 'shipping', 'html')) {
    $shipping_label = t('Shipping to');
  }

  if ($billing_to = theme('store_format_address', $txn, 'billing', 'html')) {
    $billing_label = t('Billing to');
  }

  if (!empty($txn->ship)) {
    $shipping_method_label = t('Shipping method:');
    $shipping_method = store_format_shipping_method($txn);
  }
  $email_label = t('E-mail:');
  $items_label = t('Items ordered');
  $items_view = theme('table', $header, $row, array('cellpadding' => 3, 'cellspacing' => 3));

  $payment_label = t('Payment Info');

  if (!empty($print_mode)) {
    $output .= <<<EOD
<html>
  <head>
    <style type="text/css" media="all">@import url('$css');</style>
  </head>
  <body>
EOD;
  }

$output .= <<<EOD
    <h1>$site_name</h1>

    <table cellspacing="5">
      <tr>
        <th align="left">$shipping_label</th>
        <th align="left">$billing_label</th>
      </tr>
      <tr>
        <td>$shipping_to</td>
        <td>$billing_to</td>
      </tr>
    </table>

    <p><strong>$shipping_method_label</strong> $shipping_method</p>
    <p><strong>$email_label</strong> $txn->mail</p>

    <h2>$items_label</h2>
    $items_view

    <h2>$payment_label</h2>
    $payment_info
EOD;

if (!empty($print_mode)) {
  $output .= <<<EOD
    </body>
  </html>
EOD;
}

  if (empty($print_mode)) {
    return $output;
  }
  echo $output;
}

/**
 * Called by store_transaction_form()
 */
function _store_transaction_form($edit = null) {
  $edit = is_array($edit) ? (object) $edit : $edit;

  $section = $edit->section ? $edit->section : $_GET['section'];

  switch (strtoupper($section)) {

    case 'OVERVIEW':
      $form = store_transaction_overview_form($edit);
      break;

    case 'ADDRESSES':
      $form = store_transaction_addresses_form($edit);
      break;

    case 'ITEMS':
      $form = store_transaction_item_form($edit);
      break;

    default:
      $form = array_merge(store_transaction_overview_form($edit), store_transaction_addresses_form($edit), store_transaction_item_form($edit));
  }

  if ($edit->uid && is_numeric($edit->uid)) {
    $edit->uid = db_result(db_query('SELECT name FROM {users} WHERE uid = %d', $edit->uid));
  }

  if (!empty($edit->txnid)) {
    $form['txnid'] = array(
      '#type' => 'value',
      '#value' => $edit->txnid,
      );
    $form[] = array(
      '#type' => 'submit',
      '#value' => t('Update transaction')
      );
    $form[] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      );
  }
  else {
    $form[] = array(
      '#type' => 'submit',
      '#value' => t('Create new transaction'),
      );
  }

  $form['section'] = array(
    '#type' => 'value',
    '#value' => $section,
    );

  return $form;
}

/**
 * Called by store_admin_overview()
 */
function _store_admin_overview() {

  $num_pending    = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction} WHERE (allocation = %d OR workflow = %d) AND workflow IN (%d, %d, %d, %d)', EC_ALLOC_PENDING, EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED, EC_WORKFLOW_INVOICED, EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE));
  $num_processing = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction} WHERE allocation = %d AND workflow IN (%d, %d, %d, %d)', EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED, EC_WORKFLOW_INVOICED, EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE));
  $num_completed  = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction} WHERE allocation = %d and workflow = %d', EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED));

  /* Counter the new table width: 100% in drupal.css */
  $style = array('class' => 'store_overview');
  drupal_add_css(drupal_get_path('module', 'store') .'/store.css');

  $header = array(array('data' => t('Orders'), 'colspan' => 2));
  $row[] = array(array('data' => l(t('Payments pending'), 'admin/store/transaction', array(), 'allocation=0,1&workflow=1,2,3,4'), 'align' => 'right', 'title' => t('Payment status: Pending, workflow: Not Complete')), $num_pending);
  $row[] = array(array('data' => l(t('Orders needing action'), 'admin/store/transaction', array(), 'allocation=2&workflow=1,2,3,4'), 'align' => 'right', 'title' => t('Payment status: Complete, workflow: Not Complete')), $num_processing);
  $row[] = array(array('data' => l(t('Completed'), 'admin/store/transaction', array(), 'workflow=6'), 'align' => 'right'), $num_completed);

  $output = theme('table', $header, $row, $style) .'<br />';
  $row = null;

  $beg_month   = mktime(0, 0, 0, date('m'), 1, date('Y'));
  $num_orders  = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction} where created >= %d AND allocation = %d AND workflow = %d', $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED));
  $mon_sales   = format_currency(db_result(db_query('SELECT SUM(gross) FROM {ec_transaction} where created >= %d AND allocation = %d AND workflow = %d', $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED)));

  $header = array(array('data' => t('Monthly figures'), 'colspan' => 2));
  $row[] = array(array('data' => l(t('Monthly orders'), 'admin/store/transaction', array(), 'date=>'. date('m/d/Y', $beg_month)), 'align' => 'right'), $num_orders);
  $row[] = array(array('data' => t('Monthly sales'), 'align' => 'right'), $mon_sales);

  $output .= theme('table', $header, $row, $style) .'<br />';
  $row = null;

  if (module_exists('ec_recurring')) {
    $num_active  = ec_recurring_num_product_active();
    $header = array(array('data' => t('Recurring Products'), 'colspan' => 2));
    $row[] = array(array('data' => l(t('Active products'), 'store/recurring/products/active', array(), 'nid_expires=now&workflow=6'), 'align' => 'right'), $num_active);
    $output .= theme('table', $header, $row, $style) .'<br />';
    $row = null;
  }

  $beg_year       = mktime(0, 0, 0, 1, 1, date('Y'));
  $num_customers  = db_result(db_query('SELECT COUNT(DISTINCT uid) FROM {ec_transaction}'));
  $num_products   = db_result(db_query('SELECT COUNT(nid) FROM {ec_product}'));
  $year_sales     = format_currency(db_result(db_query('SELECT SUM(gross) FROM {ec_transaction} where created >= %d AND allocation = %d AND workflow = %d', $beg_year, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED)));

  $header = array(array('data' => t('General statistics'), 'colspan' => 2));
  $row[] = array(array('data' => l(t('Customers'), 'admin/user'), 'align' => 'right'), $num_customers);
  $row[] = array(array('data' => l(t('Products'), 'product'), 'align' => 'right'), $num_products);
  $row[] = array(array('data' => t('Yearly sales'), 'align' => 'right'), $year_sales);

  /*
  // Started to work on some queries to generate top selling product reports.
  $result = db_query('SELECT nid, title, COUNT(*) as count FROM {ec_transaction_product} GROUP BY nid ORDER BY count DESC');
  $header = array(array('data' => 'Product statistics', 'colspan' => 2));
  $row[] = array(array('data' => l(t('Top selling'), 'products/top'), 'align' => 'right'), $num_customers);
  */

  $output .= theme('table', $header, $row, $style);
  $row = null;

  return $output;
}

/**
 * Called by store_transactions_search_form()
 *
 * FIXME: Problem with #multiform and setting up the args correctly to be
 * able to use urls
 */
function _store_transactions_search_form($query, $form_values = null) {
  drupal_add_css(drupal_get_path('module', 'store'). '/store.css', 'module');
  if (empty($form_values)) {
    $form_values = $query;
  }
  $form['#multistep'] = true;
  $form['#redirect'] = false;
  $form['basic'] = array('#prefix' => '<div class="container-inline">', '#suffix' => '</div>');
  $form['basic']['text'] = array(
    '#type' => 'textfield',
    '#title' => t('Search for'),
    '#default_value' => empty($form_values['text']) ? '' : $form_values['text'],
    '#size' => 60,
    '#maxlength' => 255
  );
  $form['basic']['submit'] = array('#type' => 'submit', '#value' => t('Search'));
  $form['instructions'] = array(
    '#type' => 'markup',
    '#prefix' => '<div class="description">',
    '#suffix' => '</div>',
    '#value' => t('Searches the title and content of a product for a string of text.')
  );
  $form['advanced'] =  array(
    '#type' => 'fieldset',
    '#title' => t('Advanced search'),
    '#description' => '<p>'. t('The textfields in this form recognize special operators like greater than (&gt;), less than (&lt;), comma separated values and equals to, which is also the default option if none is specified. This allows you to easily find ranges and multiple values and works with IDs and dates.') .'</p>',
    '#collapsible' => true,
    '#collapsed' => true,
    '#attributes' => array('class' => 'search-advanced')
  );
  $form['advanced']['fields'] = store_transactions_advanced_search_form($form_values);
  $form['#form_values'] = isset($form_values) ? $form_values : array();

  if (isset($form_values)) {
    $form['results'] = array(
      '#value' => store_transactions_search_result(store_transactions_search_form_parse($form_values)),
      '#weight' => 99,
    );
  }
  return $form;
}

/**
 * Called by store_transactions_advanced_search_form()
 */
function _store_transactions_advanced_search_form($form_values) {
  $form['keywords1'] = array('#prefix' => '<div class="criterion">', '#suffix' => '</div>');
  $form['keywords1']['txnid'] = array(
    '#type' => 'textfield',
    '#title' => t('Transaction ID'),
    '#default_value' => empty($form_values['txnid']) ? '' : $form_values['txnid'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('The ID number of a transaction.')
  );
  $form['keywords1']['uid'] = array(
    '#type' => 'textfield',
    '#title' => t('Username or ID'),
    '#default_value' => empty($form_values['uid']) ? '' : $form_values['uid'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('The user that placed the order.')
  );
  $form['keywords1']['nid'] = array(
    '#type' => 'textfield',
    '#title' => t('Product ID'),
    '#default_value' => empty($form_values['nid']) ? '' : $form_values['nid'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Find a transaction that has the following product(s).')
  );
  $form['keywords2'] = array('#prefix' => '<div class="criterion">', '#suffix' => '</div>');
  $form['keywords2']['nid_expires'] = array(
    '#type' => 'textfield',
    '#title' => t('Product expiration'),
    '#default_value' => empty($form_values['nid_expires']) ? '' : $form_values['nid_expires'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Date product expires.')
  );
  $form['keywords2']['date'] = array(
    '#type' => 'textfield',
    '#title' => t('Date created'),
    '#default_value' => empty($form_values['date']) ? '' : $form_values['date'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Date transaction was created.')
  );
  $form['keywords2']['duedate'] = array(
    '#type' => 'textfield',
    '#title' => t('Date due'),
    '#default_value' => empty($form_values['duedate']) ? '' : $form_values['duedate'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Date transaction was due.')
  );
  $form['keywords2']['gross'] = array(
    '#type' => 'textfield',
    '#title' => t('Total cost'),
    '#default_value' => empty($form_values['gross']) ? '' : $form_values['gross'],
    '#size' => 30,
    '#maxlength' => 255
  );
  /**
   * TODO: Change to use the new allocation status
  $form['status'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Payment status'),
    '#default_value' => empty($form_values['status']) ? '' : $form_values['status'],
    '#options' => payment_get_states(),
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  ); */
  $form['workflow'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Workflow'),
    '#default_value' => empty($form_values['workflow']) ? '' : $form_values['workflow'],
    '#options' => store_transaction_workflow(),
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  );
  /**
   * TODO: Change to use ec_receipts
   *
  $form['method'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Payment method'),
    '#default_value' => empty($form_values['method']) ? '' : $form_values['method'],
    '#options' => payment_get_methods(),
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  );
   */
  $form['ptype'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Product type'),
    '#default_value' => empty($form_values['ptype']) ? '' : $form_values['ptype'],
    '#options' => product_get_ptypes('names'),
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Advanced search'),
    '#prefix' => '<div class="action">',
    '#suffix' => '</div>',
    '#weight' => 99,
  );
  return $form;
}

/**
 * Called by store_transactions_search_result()
 */
function _store_transactions_search_result($selection, $show_details = true, $show_permlink = true) {
  require_once(drupal_get_path('module', 'store') .'/ec_query.inc');

  static $pq_element = 0;
  $output = "";
  // If we have a valid query object, let's hit the database.
  if (!empty($selection)) {

    $query = new _ec_query();
    $query->is_distinct();

    foreach ($selection as $criterion => $criterion_value) {
      $data = array();
      $operator = null;
      $criterion_value = trim($criterion_value);

      // Handle any comparison operators
      if (strpos($criterion_value, ',') !== false) {
        $operator = ',';
        $data     = explode(',', $criterion_value);
        $data = array_map('db_escape_string', $data);
      }
      elseif (strpos($criterion_value, '-') !== false) {
        $operator = '-';
        $data     = explode('-', $criterion_value);
        $data = array_map('db_escape_string', $data);
      }
      else {
        // This order is important.
        $operators = array('>=', '<=', '>', '<', '=');
        foreach ($operators as $op) {
          $op_len = strlen($op);
          // Only catch these operators at the beginning of the value.
          if (strlen($criterion_value) >= $op_len && substr($criterion_value, 0, $op_len) == $op) {
            $operator = '<>';
            $data['op'] = $op;
            $data[0] = db_escape_string(trim(substr($criterion_value, $op_len)));
            break;
          }
        }
      }

      if ($operator == null) {
        $data[] = db_escape_string($criterion_value);
      }

      switch ($criterion) {
        case 'text':
          $query->ensure_table('ec_transaction_product', 'stp', 'INNER', 'st.txnid = stp.txnid');
          $query->ensure_table('node', 'n', 'LEFT', 'stp.vid = n.vid');
          $query->ensure_table('node_revisions', 'r', 'LEFT', 'stp.vid = r.vid');
          $query->add_where("(n.title LIKE '%%{$data[0]}%%' OR r.body LIKE '%%{$data[0]}%%')");
          $details[] = _store_transactions_search_detail(t('Text'), $data, $operator);
          break;

        case 'txnid':
          $query->add_where(_store_transactions_search_operator($data, $operator, 'st.txnid'));
          $details[] = _store_transactions_search_detail(t('Transaction id'), $data, $operator);
          break;

        case 'uid':
          $query->ensure_table('users', 'u', 'LEFT', 'st.uid = u.uid');

          $where_uid = '';
          $op = ($data['op']) ? $data['op'] : '=';
          foreach ($data as $key => $value) {
            if (is_numeric($value)) {
              $where_uid .= "OR st.uid $op '$value'";
            }
            else {
              $where_uid .= "OR u.name LIKE '%%$value%%'";
            }
            ++$i;
          }
          $where_uid .= ')';

          $query->add_where($where_uid);
          $details[] = _store_transactions_search_detail(t('User'), $data, $operator);
          break;

        case 'nid':
          $query->ensure_table('ec_transaction_product', 'stp', 'INNER', 'st.txnid = stp.txnid');
          $query->ensure_table('ec_product', 'p', 'INNER', 'stp.vid = p.vid');
          $query->add_where(_store_transactions_search_operator($data, $operator, 'p.nid'));
          $details[] = _store_transactions_search_detail(t('Product id'), $data, $operator);
          break;

        case 'nid_expires':
          $query->ensure_table('ec_transaction_product', 'stp', 'INNER', 'st.txnid = stp.txnid');
          $query->ensure_table('ec_product', 'p', 'INNER', 'stp.vid = p.vid');
          $mod = array();
          foreach ($data as $key => $value) {
            if ($key !== 'op') {
              $mod[$key] = format_date(strtotime($value), 'small');
            }
          }
          $query->add_where(_store_transactions_search_operator($data, $operator, 'stp.expires', 'strtotime'));
          $details[] = _store_transactions_search_detail(t('Product expires'), $mod, $operator);
          break;

        case 'date':
          $mod = array();
          foreach ($data as $key => $value) {
            if ($key !== 'op') {
              $mod[$key] = format_date(strtotime($value), 'small');
            }
          }
          $query->add_where(_store_transactions_search_operator($data, $operator, 'st.created', 'strtotime'));
          $details[] = _store_transactions_search_detail(t('Date'), $mod, $operator);
          break;

        case 'duedate':
          $mod = array();
          foreach ($data as $key => $value) {
            if ($key !== 'op') {
              $mod[$key] = format_date(strtotime($value), 'small');
            }
          }
          $query->add_where(_store_transactions_search_operator($data, $operator, 'st.duedate', 'strtotime'));
          $query->add_where('AND st.duedate != 0');
          $details[] = _store_transactions_search_detail(t('Date Due'), $mod, $operator);
          break;

        case 'gross':
          $query->add_where(_store_transactions_search_operator($data, $operator, 'st.gross'));
          $details[] = _store_transactions_search_detail(t('Cost'), $data, $operator);
          break;

        case 'allocation':
          $mod = array();
          foreach ($data as $key => $value) {
            $value = ($key !== 'op') ? transaction_get_workflow($value) : $value;
            $mod[$key] = $value;
          }
          $query->add_where(_store_transactions_search_operator($data, $operator, 'st.workflow'));
          $details[] = _store_transactions_search_detail(t('Allocation'), $mod, $operator);
          break;

        case 'workflow':
          $mod = array();
          foreach ($data as $key => $value) {
            $value = ($key !== 'op') ? transaction_get_workflow($value) : $value;
            $mod[$key] = $value;
          }
          $query->add_where(_store_transactions_search_operator($data, $operator, 'st.workflow'));
          $details[] = _store_transactions_search_detail(t('Workflow'), $mod, $operator);
          break;

        case 'methodxxx':
          $methods = payment_get_methods();
          $mod = array();
          foreach ($data as $key => $value) {
            $value = ($key !== 'op') ? $methods[$value] : $value;
            $mod[$key] = $value;
          }
          $query->add_where(_store_transactions_search_operator($mod, $operator, 'st.payment_method'));
          $details[] = _store_transactions_search_detail(t('Payment method'), $mod, $operator);
          break;

        case 'ptype':
          $query->ensure_table('ec_transaction_product', 'stp', 'INNER', 'st.txnid = stp.txnid');
          $query->ensure_table('ec_product', 'p', 'INNER', 'stp.vid = p.vid');
          $query->add_where(_store_transactions_search_operator($data, $operator, 'p.ptype'));
          $details[] = _store_transactions_search_detail(t('Product type'), $data, $operator);
          break;

        default:
          foreach (module_implements('ec_transaction_search') as $module) {
            $f = $module .'_ec_transaction_search';
            $f($query, $criterion, $operator, $data, $details);
          }
      }
      $get[] = _store_transactions_search_uri($criterion, $data);
    } //The end of a really really long for loop.

    $output .= '<div class="transactions">'. store_query_output('transaction_admin_search', $criterion, $query, $show_details ? $details :  array()) .'</div>';
    $output .= '<div id="help"><strong>*</strong>'.
      t('Clicking links marked with an asterisk will also cause the transaction to shift into it\'s respective workflow state.') .
      '</div>';
    if (!empty($show_permlink)) {
      $output .= '<p>'. l('#', 'admin/store/transaction', array('title' => t('Permanent link to this search query.')), implode('&', $get)) .'</p>';
    }
  }

  ++$pq_element;
  return $output;
}
