<?php
/**
 * $Id: subproducts.admin.inc,v 1.1.2.6 2007/08/13 20:11:51 brmassa Exp $
 * @author Bruno Massa http://drupal.org/user/67164
 * @file subproducts.admin.inc
 * All manage pages and functions are now in a separated file
 * to minimize the module footprint.
 */

/**
 * View all variations and attributes
 *
 * From here the admin can create, update or delete
 * variations and attributes to be used on parent
 * products.
 *
 * @return
 *   Array, form with all variations and attributes
 */
function _subproducts_attribute_admin() {
  $items = array();
  $current_variation = null;
  $variations = subproducts_attribute_get(null, null, 50);
  while (list(, $variation) = each($variations)) {
    $item_temp['variation'] = l($variation['name'], 'admin/content/taxonomy/edit/term/'. $variation['tid'], null,
        'destination=admin/store/attributes');
    array_push($items, $item_temp);
    $item_temp = array();
    while (list($nid, $attribute) = each($variation['attr'])) {
      $item_temp['attribute'] = l($attribute->title, 'node/'. $attribute->nid .'/edit', null,
          'destination=admin/store/attributes&variation='. $variation['tid']);
      $item_temp['price'] = format_currency($attribute->price);
      if (product_feature_enabled($attribute->ptype, 'shippable')) {
        $item_temp['stock'] = $attribute->stock;
      }
      $item_temp['sku'] = $attribute->sku;
      array_push($items, $item_temp);
    }
  }
  return theme('subproducts_attribute_admin', $items);
}

/**
 * Show a list of all attributes to be associated to this parent product
 * @ingroup forms
 * @see _subproducts_pproduct_attributes_submit()
 * @see _subproducts_pproduct_attributes_validate()
 */
function _subproducts_pproduct_attributes() {
  $current_variation = null;
  $form = array();
  $node = node_load(arg(1));
  if (!($variations = subproducts_attribute_get())) {
    return array();
  }
  while (list(, $variation) = each($variations)) {
    $default = null;
    if (empty($variation['attr'])) {
      continue;
    }
    while (list(, $attribute) = each($variation['attr'])) {
      if (empty($default)) {
        $default = empty($node->default_selection[$variation['tid']]) ?
          $attribute->nid : $node->default_selection[$variation['tid']];
      }
      $form['include'][$attribute->nid] = array(
        '#type'           => 'checkbox',
        '#default_value'  => !empty($node->sp_attributes[$variation['tid']]['attr'][$attribute->nid])
      );
      $form['variation'][$attribute->nid] = array(
        '#type'           => 'item',
        '#value'          => $variation['name']
      );
      $form['pricevariation'][$attribute->nid] = array(
        '#type'           => 'item',
        '#value'          => format_currency($attribute->price)
      );
      $form['stock'][$attribute->nid] = array(
        '#type'           => 'item',
        '#value'          => $attribute->stock
      );
      $form['sku'][$attribute->nid] = array(
        '#type'           => 'item',
        '#value'          => $attribute->sku
      );
      $form['default'][$attribute->nid] = array(
        '#type'           => 'radio',
        '#title'          => $attribute->title,
        '#parents'        => array($variation['tid']),
        '#return_value'   => $attribute->nid,
        '#default_value'  => $default
      );
      $form['variation_hidden'][$attribute->nid] = array(
        '#type'           => 'hidden',
        '#value'          => $variation['name']
      );
    }
  }
  $form['sku_title'] = array(
    '#type' => 'checkbox',
    '#title' => t('Assume SKU as title'),
    '#description' => t('If your product has many variations, your final product name will be quite long. This your set the products name the same as SKU.'),
    '#default_value' => !empty($node->sku_title) ? $node->sku_title : 0
  );
  $form['nid'] = array('#type' => 'hidden', '#value' => arg(1));
  $form['#tree'] = true;
  $form['submit'] = array(
    '#type'   => 'submit',
    '#value'  => t('Submit')
  );
  return $form;
}

/**
 * @see _subproducts_pproduct_attributes()
 */
function _subproducts_pproduct_attributes_submit($form_id, &$form) {
  $node = node_load($form['nid']);
  $sql = $sql_args = $default_selection = array();
  // delete all parent product attributes and relink only the marked
  db_query('DELETE FROM {ec_sp_bpattribute} WHERE pnid = %d', $node->nid);
  foreach (element_children($form['include']) as $attribute) {
    if (!empty($form['include'][$attribute])) {
      array_push($sql, '(%d, %d)');
      array_push($sql_args, $node->nid);
      array_push($sql_args, $attribute);
    }
  }
  if (!empty($sql)) {
    db_query('INSERT INTO {ec_sp_bpattribute} (pnid, nid) VALUES '. implode(',', $sql), $sql_args);
  }
  // assing special values to the parent product only
  while (list($variation, $attribute) = each($form)) {
    if (is_numeric($variation)) {
      array_push($default_selection, $variation .','. $attribute);
    }
  }
  $default_selection = implode(';', $default_selection);
  if (!isset($node->pproduct)) {
    db_query('INSERT INTO {ec_sp_pproduct} (sku_title, default_selection, nid) VALUES (%d, "%s",%d)',
      $form['sku_title'], $default_selection, $node->nid);
  }
  else {
    db_query('UPDATE {ec_sp_pproduct} SET sku_title = %d, default_selection = "%s" WHERE nid = %d',
      $form['sku_title'], $default_selection, $node->nid);
  }
  node_save($node); // save the parent product
}

/**
 * @see _subproducts_pproduct_attributes()
 */
function _subproducts_pproduct_attributes_validate($form_id, &$form) {
  $current_variation = null;
  foreach ($form as $variation => $attribute) {
    if (is_numeric($variation)) {
      $default_selection[$attribute] = true;
    }
  }
  foreach (element_children($form['include']) as $attribute) {
    if ($current_variation != $form['variation_hidden'][$attribute]) {
      if (!empty($forbidden_default_value) and !empty($variation_set)) {
        form_set_error('submit', t('Only enabled attributes can be default.'), 'error');
      }
      $forbidden_default_value = false;
      $variation_set = false;
      $current_variation = $form['variation_hidden'][$attribute];
    }
    if (!empty($form['include'][$attribute])) {
      $variation_set = true;
    }
    elseif (empty($form['include'][$attribute]) and !empty($default_selection[$attribute])) {
      $forbidden_default_value = true;
    }
  }
  if (!empty($forbidden_default_value) and !empty($variation_set)) {
    form_set_error('submit', t('Only enabled attributes can be default.'), 'error');
  }
}

/**
 * Implementation of hook_taxonomy().
 *
 * Customise the taxomony form (terms and vocabularies) results
 */
function _subproducts_taxonomy($op, $type, $form = null) {
  if ($type == 'term' and isset($form['showprice'])) {
    if ($op == 'insert') {
      db_query('INSERT INTO {ec_sp_variation} (listtype,showprice,tid) VALUES (%d,%b,%d)',
        $form['listtype'], $form['showprice'], $form['tid']);
    }
    elseif ($op == 'update') {
      db_query('UPDATE {ec_sp_variation} SET listtype = %b, showprice = %b WHERE tid = %d',
        $form['listtype'], $form['showprice'], $form['tid']);
    }
  }
}

/**
 * Print all attributes (admin)
 *
 * @param &$items
 *   Array, the form including the parent product attributes
 * @return
 *   String, HMTL with attributes (admin)
 * @ingroup themeable
 */
function theme_subproducts_attribute_admin(&$items) {
  $current_variation = '';
  $header = array(t('Variation') .'/'. t('Attribute'), t('Price Variation'), t('Stock'), t('SKU'));
  $rows = array();
  foreach (element_children($items) as $attribute) {
    if (!empty($items[$attribute]['variation']) and
        $current_variation != $items[$attribute]['variation']) {
      $current_variation = $items[$attribute]['variation'];
      array_push($rows, array(
        array('data' => '<b>'. $items[$attribute]['variation'] .'</b>', 'colspan' => 4),
      ));
    }
    if (!empty($items[$attribute]['attribute']['#value'])) {
      array_push($rows, array(
        $items[$attribute]['attribute'],
        $items[$attribute]['price'],
        $items[$attribute]['stock'],
        $items[$attribute]['sku']
      ));
    }
  }
  if (!empty($rows)) {
    $pager = theme('pager', null, 50, 0);
    if (!empty($pager)) {
      array_push($rows, array(array('data' => $pager, 'colspan' => 6)));
    }
    $output = theme('table', $header, $rows);
  }
  else {
    $output = '<p>'. t('There is no variations or attributes.') .'</p>';
  }

  return $output;
}

/**
 * Print the parent product attributes (admin)
 *
 * @param &$form
 *   Array, the form including the parent product attributes
 * @return
 *   String, HMTL with the parent product attributes (admin)
 * @ingroup themeable
 * @see theme_subproducts_pproduct_attributes()
 */
function theme_subproducts_pproduct_attributes_admin(&$form) {
  $current_variation = null;
  foreach (element_children($form['include']) as $attribute) {
    if (!empty($form['variation'][$attribute]['#value']) and $current_variation != $form['variation'][$attribute]['#value']) {
      $current_variation = $form['variation'][$attribute]['#value'];
      $rows[] = array(
        array('data' => '<em>'. drupal_render($form['variation'][$attribute]) .'</em>', 'colspan' => 5),
      );
    }
    unset($form['variation'][$attribute]);
    $rows[] = array(
      drupal_render($form['include'][$attribute]),
      drupal_render($form['default'][$attribute]),
      drupal_render($form['pricevariation'][$attribute]),
      drupal_render($form['stock'][$attribute]),
      drupal_render($form['sku'][$attribute])
    );
  }
  if (!empty($rows)) {
    $pager = theme('pager', null, 50, 0);
    if (!empty($pager)) {
      $rows[] = array(array('data' => $pager, 'colspan' => 6));
    }
    $header = array(theme('table_select_header_cell'), t('Default'), t('Price Variation'), t('Stock'), t('SKU'));
    $output = drupal_render($form['sku_title']);
    $output .= drupal_render($form['submit']);
    $output .= theme('table', $header, $rows);
    $output .= drupal_render($form);
  }
  else {
    $output = '<p>'. t('There is no variations or attributes.') .'</p>';
  }
  return $output;
}
